/*------------------------------------------------*/
/* 08textury/textury/main.c                       */
#define _CRT_SECURE_NO_WARNINGS
#define GLEW_STATIC
#include <GL/glew.h>
#include <stdio.h>
#include <GLFW/glfw3.h>
#include <stdbool.h>
#include <math.h>
#include "linmath-extended.h"
#include "vertex-data.h"
#include "shader.h"
#include "update-angles.h"
#include "events.h"
#include "stb-image.h"
#include "load-cubemap.h"

void initVariables(Shader *program);
void initBuffers(Shader *program);
void uninitBuffers(void);
void ticking(Shader * program, GLFWwindow* window);
void draw(Shader *program, GLFWwindow* window, GLuint *VAO);
void animate(void);

GLuint VAO[1], BO[2], TEX[1];
struct {
	GLuint uPMatrixLoc;
	GLuint uMVMatrixLoc;
	GLuint uAmbientColorLoc;
	GLuint texture1;
	GLuint useAmbientLoc;
} variables;
mat4x4 rotationMatrix;

int main(void)
{
	GLFWwindow* window;
	/* Initialize the library */
	if (!glfwInit()) {
		return -1;
	}
	glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3);
	glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 3);
	glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);

	/* Create a windowed mode window and its OpenGL context */
	GLFWmonitor* monitor = glfwGetPrimaryMonitor(); 	
	const GLFWvidmode* mode = glfwGetVideoMode(monitor);
	glfwWindowHint(GLFW_RED_BITS, mode->redBits);
	glfwWindowHint(GLFW_GREEN_BITS, mode->greenBits);
	glfwWindowHint(GLFW_BLUE_BITS, mode->blueBits);
	glfwWindowHint(GLFW_REFRESH_RATE, mode->refreshRate);
	window = glfwCreateWindow(mode->width, mode->height, "08 - cubemap", monitor, NULL);
	if (!window)
	{
		printf("Nelze inicializovat kontext\n");
		glfwTerminate();
		return -1;
	}
	/* Make the window's context current */
	glfwMakeContextCurrent(window);
	printf("%s\n", glGetString(GL_VERSION));

	/* Load modern OpenGL functions */
	glewExperimental = GL_TRUE;
	glewInit();

	/* events */
	glfwSetKeyCallback(window, key_callback);
	glfwSetMouseButtonCallback(window, mouse_button_callback);
	glfwSetCursorPosCallback(window, cursor_position_callback);

	//glfwSwapInterval(1);

	Shader program = shader("cubemap.vs", "cubemap.fs");
	if (!program.Program) {
		return -1;
	}

	initVariables(&program);
	initBuffers(&program);

	// setup GL state
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	mat4x4_identity(rotationMatrix);
	useColor.ambient = 0;

	ticking(&program, window);

	uninitBuffers();

	glfwDestroyWindow(window);
	glfwTerminate();
	return 0;
}

void initVariables(Shader *program)
{
	variables.uMVMatrixLoc = program->getUniformLocation(program, "uMVMatrix");
	variables.uPMatrixLoc = program->getUniformLocation(program, "uPMatrix");
	variables.uAmbientColorLoc = program->getUniformLocation(program, "uAmbientColor");
	variables.texture1 = program->getUniformLocation(program, "texture1");
	variables.useAmbientLoc = program->getUniformLocation(program,"useAmbient");
}

void initBuffers(Shader *program)
{
	glGenVertexArrays(sizeof(VAO) / sizeof(VAO[0]), VAO);
	glGenBuffers(sizeof(BO) / sizeof(BO[0]), BO);
	glGenTextures(sizeof(TEX) / sizeof(TEX[0]), TEX);

	const char * const faces[] = {
		"miramar_large/right.jpg",
		"miramar_large/left.jpg",
		"miramar_large/top.jpg",
		"miramar_large/bottom.jpg",
		"miramar_large/back.jpg",
		"miramar_large/front.jpg",
	};
	//*/
	/*
	const char * const faces[] = {
		"mp_amh/amh_ft.tga",
		"mp_amh/amh_bk.tga",
		"mp_amh/amh_up.tga",
		"mp_amh/amh_dn.tga",
		"mp_amh/amh_rt.tga",
		"mp_amh/amh_lf.tga",
	};
	**/
	glActiveTexture(GL_TEXTURE0);
	load_cubemap(TEX[0], faces);

	glBindVertexArray(VAO[0]);
	{
		//vertices
		glBindBuffer(GL_ARRAY_BUFFER, BO[0]);
		glBufferData(GL_ARRAY_BUFFER, sizeof(boxVertices), boxVertices, GL_STATIC_DRAW);
		glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 3 * sizeof(GLfloat), (GLvoid*)0);
		glEnableVertexAttribArray(0);
		//textures
		glBindBuffer(GL_ARRAY_BUFFER, BO[1]);
		glBufferData(GL_ARRAY_BUFFER, sizeof(boxTexCoord), boxTexCoord, GL_STATIC_DRAW);
		glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, 3 * sizeof(GLfloat), (GLvoid*)0);
		glEnableVertexAttribArray(1);
	}
	glBindVertexArray(0); // Unbind VAO
	glBindBuffer(GL_ARRAY_BUFFER, 0);
}

void uninitBuffers(void)
{
	glDeleteVertexArrays(sizeof(VAO) / sizeof(VAO[0]), VAO);
	glDeleteBuffers(sizeof(BO) / sizeof(BO[0]), BO);
}

void ticking(Shader * program, GLFWwindow* window) {
	/* Loop until the user closes the window */
	while (!glfwWindowShouldClose(window))
	{
		animate();
		draw(program, window, VAO);
		glfwSwapBuffers(window);
		glfwPollEvents();
	}
}

void animate(void) {
}


void draw(Shader *program, GLFWwindow* window, GLuint *VAO)
{
	int width, height;
	glfwGetFramebufferSize(window, &width, &height);
	glViewport(0, 0, width, height);

	glClearColor(1.f, 1.f, 1.f, 1.f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	program->Use(program);

	mat4x4 uMVMatrix, uPMatrix;

	mat4x4_identity(uMVMatrix);
	mat4x4_identity(uPMatrix);
	mat4x4_perspective(uPMatrix, degToRad(90.0), ((GLfloat)width) / height, 1.0, 2000.0);
	//mat4x4_translate(uMVMatrix, 0.0, 0.0, -5.0);
	//mat4x4_scale_aniso(uMVMatrix, uMVMatrix, 0.002, 0.002, 0.002);
	mat4x4_mul(uMVMatrix, uMVMatrix, rotationMatrix);

	glUniformMatrix4fv(variables.uMVMatrixLoc, 1, GL_FALSE, (const GLfloat *)uMVMatrix);
	glUniformMatrix4fv(variables.uPMatrixLoc, 1, GL_FALSE, (const GLfloat *)uPMatrix);
	glUniform3f(variables.uAmbientColorLoc, 0.4, 0.4, 0.4);
	glUniform1i(variables.useAmbientLoc, useColor.ambient);

	// vykresli skybox
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_CUBE_MAP, TEX[0]);
	glBindVertexArray(VAO[0]);
	glDrawArrays(GL_TRIANGLES, 0, sizeof(boxVertices) / sizeof(boxVertices[0]) / 3);
	glBindVertexArray(0);
	glBindTexture(GL_TEXTURE_CUBE_MAP, 0);
	glActiveTexture(0);
}

/*------------------------------------------------*/
