/*------------------------------------------------*/
/* 08textury/teren/main.c                         */
#define _CRT_SECURE_NO_WARNINGS
#define GLEW_STATIC
#include <GL/glew.h>
#include <stdio.h>
#include <GLFW/glfw3.h>
#include <stdbool.h>
#include <math.h>
#include "linmath-extended.h"
#include "ground.h"
#include "shader.h"
#include "normals.h"
#include "update-angles.h"
#include "events.h"
#include "stb-image.h"

void initVariables(Shader *program);
void initBuffers(Shader *program, Ground ground);
void uninitBuffers(Ground ground);
void ticking(Shader * program, GLFWwindow* window, Ground ground);
void draw(Shader *program, GLFWwindow* window, GLuint *VAO, Ground ground);
void animate();

GLuint VAO[1], BO[3], TEX[1];
struct {
	GLuint uPMatrixLoc;
	GLuint uMVMatrixLoc;
	GLuint uNMatrixLoc;
	GLuint uAmbientColorLoc;
	GLuint uUseDirectionalLoc;
	GLuint uDirectionalColorLoc;
	GLuint uLightingPositionLoc;
	GLuint uUseSpecularLoc;
	GLuint texture1;
} variables;
mat4x4 rotationMatrix;
float x, y, z; // light position

int main(void)
{
	GLFWwindow* window;
	/* Initialize the library */
	if (!glfwInit()) {
		return -1;
	}
	glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3);
	glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 3);
	glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);

	/* Create a windowed mode window and its OpenGL context */
	/* Create a windowed mode window and its OpenGL context */
	GLFWmonitor* monitor = glfwGetPrimaryMonitor(); 	
	const GLFWvidmode* mode = glfwGetVideoMode(monitor);
	glfwWindowHint(GLFW_RED_BITS, mode->redBits);
	glfwWindowHint(GLFW_GREEN_BITS, mode->greenBits);
	glfwWindowHint(GLFW_BLUE_BITS, mode->blueBits);
	glfwWindowHint(GLFW_REFRESH_RATE, mode->refreshRate);
	window = glfwCreateWindow(mode->width, mode->height, "08 - teren", monitor, NULL);
	if (!window)
	{
		printf("Nelze inicializovat kontext\n");
		glfwTerminate();
		return -1;
	}
	/* Make the window's context current */
	glfwMakeContextCurrent(window);
	printf("%s\n", glGetString(GL_VERSION));

	/* Load modern OpenGL functions */
	glewExperimental = GL_TRUE;
	glewInit();

	/* events */
	glfwSetKeyCallback(window, key_callback);
	glfwSetMouseButtonCallback(window, mouse_button_callback);
	glfwSetCursorPosCallback(window, cursor_position_callback);

	//glfwSwapInterval(1);

        Shader program = shader("default.vs", "default.fs");
	if(!program.Program) {
		return -1;
	}

	const int f = 5.0;
	Ground ground = generateGround(100*f, 100*f, 1.5/f, 1.5/f, 0.5*f);
	initVariables(&program);
	initBuffers(&program, ground);

	// setup GL state
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	mat4x4_identity(rotationMatrix);
	mat4x4_rotate_X(rotationMatrix, rotationMatrix, 3.14159/10.0);
	useColor.ambient = 1;
	useColor.directional = 1;
	useColor.specular = 1;

	ticking(&program, window, ground);

	uninitBuffers(ground);

	glfwDestroyWindow(window);
	glfwTerminate();
	return 0;
}

void initVariables(Shader *program)
{
	variables.uMVMatrixLoc = program->getUniformLocation(program, "uMVMatrix");
	variables.uPMatrixLoc = program->getUniformLocation(program, "uPMatrix");
	variables.uNMatrixLoc = program->getUniformLocation(program, "uNMatrix");
	variables.uAmbientColorLoc = program->getUniformLocation(program, "uAmbientColor");
	variables.uUseDirectionalLoc = program->getUniformLocation(program, "uUseDirectional");
	variables.uDirectionalColorLoc = program->getUniformLocation(program, "uDirectionalColor");
	variables.uLightingPositionLoc = program->getUniformLocation(program, "uLightingPosition");

	variables.uUseSpecularLoc = program->getUniformLocation(program, "uUseSpecular");
	variables.texture1 = program->getUniformLocation(program, "texture1");
}

void initBuffers(Shader *program, Ground ground)
{
        glGenVertexArrays(sizeof(VAO)/sizeof(VAO[0]), VAO);
        glGenBuffers(sizeof(BO)/sizeof(BO[0]), BO);
	glGenTextures(sizeof(TEX)/sizeof(TEX[0]), TEX);

	glBindTexture(GL_TEXTURE_2D, TEX[0]);
	//glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);   // Set texture wrapping to GL_REPEAT
	//glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);


	int width, height, n;
	unsigned char *image = sx_stbi_load("grass2.jpg", &width, &height, &n, 0);
	printf("w = %i, h = %i, n = %i\n", width, height, n);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, width, height, 0, GL_RGB, GL_UNSIGNED_BYTE, image);
	sx_stbi_image_free(image);
	//glGenerateMipmap(GL_TEXTURE_2D);
        glBindTexture(GL_TEXTURE_2D, 0);

	glBindVertexArray(VAO[0]);
	{
		//vertices
		glBindBuffer(GL_ARRAY_BUFFER, BO[0]);
	        glBufferData(GL_ARRAY_BUFFER, ground.length, ground.vertices, GL_STATIC_DRAW);
		glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 3 * sizeof(GLfloat), (GLvoid*)0);
	        glEnableVertexAttribArray(0);
		//normals
		ComputedNormals normals = createNormals(ground.vertices, ground.length/sizeof(ground.vertices[0]), NULL);
		glBindBuffer(GL_ARRAY_BUFFER, BO[1]);
	        glBufferData(GL_ARRAY_BUFFER, sizeof(GLfloat)*normals.length, normals.normals, GL_STATIC_DRAW);
		glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, 3 * sizeof(GLfloat), (GLvoid*)0);
	        glEnableVertexAttribArray(1);
		normals.free(&normals);
		//textures
		glBindBuffer(GL_ARRAY_BUFFER, BO[2]);
	        glBufferData(GL_ARRAY_BUFFER, ground.texLength, ground.texCoord, GL_STATIC_DRAW);
		glVertexAttribPointer(2, 2, GL_FLOAT, GL_FALSE, 2 * sizeof(GLfloat), (GLvoid*)0);
	        glEnableVertexAttribArray(2);
	}
	glBindVertexArray(0); // Unbind VAO
	glBindBuffer(GL_ARRAY_BUFFER, 0);
}

void uninitBuffers(Ground ground)
{
	glDeleteVertexArrays(sizeof(VAO)/sizeof(VAO[0]), VAO);
	glDeleteBuffers(sizeof(BO)/sizeof(BO[0]), BO);
	ground.free(&ground);
}

void ticking(Shader * program, GLFWwindow* window, Ground ground) {
	/* Loop until the user closes the window */
	while (!glfwWindowShouldClose(window))
	{
		animate();
		draw(program, window, VAO, ground);
		glfwSwapBuffers(window);
		glfwPollEvents();
	}
}

void animate() {
	x = 3.0;
	y = 10.0;
	z = -10+-30.0* sin(glfwGetTime()/10.0);
}


void draw(Shader *program, GLFWwindow* window, GLuint *VAO, Ground ground)
{
       	int width, height;
        glfwGetFramebufferSize(window, &width, &height);
        glViewport(0, 0, width, height);

	glClearColor(0.f, 0.f, 0.5f, 1.f);
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

       	program->Use(program);

	mat4x4 uMVMatrix, uPMatrix;
	mat3x3 uNMatrix;

	mat4x4_perspective(uPMatrix, degToRad(45.0), ((GLfloat) width)/height, 1.0, 100.0);
	mat4x4_translate(uMVMatrix, 0.0, 0.0, -10.0);
	mat4x4_mul(uMVMatrix, uMVMatrix, rotationMatrix);

	mat3x3_from_mat4x4(uNMatrix, uMVMatrix);
	mat3x3_invert(uNMatrix, uNMatrix);
	mat3x3_transpose(uNMatrix, uNMatrix);

        glUniformMatrix4fv(variables.uMVMatrixLoc, 1, GL_FALSE, (const GLfloat *) uMVMatrix);
        glUniformMatrix4fv(variables.uPMatrixLoc, 1, GL_FALSE, (const GLfloat *) uPMatrix);
        glUniformMatrix3fv(variables.uNMatrixLoc, 1, GL_FALSE, (const GLfloat *) uNMatrix);

	if(useColor.ambient) {
	        glUniform3f(variables.uAmbientColorLoc, 0.3, 0.3, 0.3);
	} else {
	        glUniform3f(variables.uAmbientColorLoc, 0.0, 0.0, 0.0);
	}
        glUniform1i(variables.uUseDirectionalLoc, useColor.directional);
	glUniform3f(variables.uDirectionalColorLoc, 0.8, 0.8, 0.8);

	glUniform3f(variables.uLightingPositionLoc, x, y, z);
        glUniform1i(variables.uUseSpecularLoc, useColor.specular);

	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, TEX[0]);
	glUniform1i(variables.texture1, 0);

	// Vykresli
        glBindVertexArray(VAO[0]);
        glDrawArrays(GL_TRIANGLES, 0, ground.length/3);
        glBindVertexArray(0);
}

/*------------------------------------------------*/
